import React, { useState, useEffect } from 'react'
import { Form, Select, Input, message, Tooltip } from 'antd'
import { getTableNamesList, getAilabList } from '../services'

const { Option } = Select

export default function LoadFormConfig({
  loadDataForm,
  dataSourceList,
  updateTableStruct,
  updateLoadAilabStruct,
}) {
  const [datasourceType, setDatasourceType] = useState(null)

  const [datasources, setDatasources] = useState()

  const [tableList, setTableList] = useState([])

  const [dsType, setDsType] = useState(null)

  // 获取数据源列表
  useEffect(() => {
    if (dataSourceList.length !== 0) {
      setDatasources(
        dataSourceList.map(item => {
          return {
            label: item.datasource_name,
            value: item.key,
          }
        })
      )
    }
  }, [dataSourceList])

  // 初始化表单类型
  useEffect(() => {
    const ds_id = loadDataForm.getFieldValue('datasource_name')
    const table_name = loadDataForm.getFieldValue('datasource_table')
    const ds_type = loadDataForm.getFieldValue('datasource_type')
    selectDsType(ds_type)
    if (ds_id) {
      const type = dataSourceList.find(item => item.key === ds_id).datasource
      setDatasourceType(type)
      selectDatasource(ds_id)
      loadDataForm.setFieldValue('datasource_table', table_name)
    }
  }, [])

  // 选择数据源
  const selectDatasource = async val => {
    loadDataForm.setFieldValue('datasource_table', null)
    const type = dataSourceList.find(item => item.key === val).datasource
    setDatasourceType(type)
    const { data } = await getTableNamesList(val)
    if (data.code === 200) {
      const list = data.data.map(item => ({
        label: item,
        value: item,
      }))
      setTableList(list)
    } else {
      message.error('数据表加载失败')
    }
  }

  const selectAilab = async () => {
    const { data } = await getAilabList()
    if (data.code === 200) {
      const list = data.data.map((item, index) => {
        return {
          key: index,
          label: item,
          value: item,
        }
      })
      setTableList(list)
    } else {
      message.error(data.msg)
    }
  }

  // 选择表
  const selectTable = val => {
    const currentTable = loadDataForm.getFieldValue('datasource_table')
    switch (dsType) {
      case 'datasource':
        const ds_id = loadDataForm.getFieldValue('datasource_name')
        updateTableStruct(ds_id, val)
        break
      default:
        updateLoadAilabStruct(currentTable)
        break
    }
  }

  const selectDsType = val => {
    setDsType(val)
    switch (val) {
      case 'datasource':
        const ds_id = loadDataForm.getFieldValue('datasource_name')
        if (ds_id) {
          selectDatasource(ds_id)
        }
        break
      default:
        setDatasourceType('hive')
        selectAilab()
        break
    }
  }

  return (
    <>
      <p
        style={{
          fontWeight: 600,
          borderLeft: '3px solid #1890ff',
          paddingLeft: '12px',
        }}>
        配置加载源
      </p>
      <Form
        name="drawForm"
        form={loadDataForm}
        labelCol={{
          span: 2,
        }}
        wrapperCol={{
          span: 6,
        }}
        initialValues={{
          pre_sql: '',
          post_sql: '',
          writer_field_delimiter: '',
        }}>
        <Form.Item
          label="数据源分类"
          name="datasource_type"
          rules={[
            {
              required: true,
              message: '请选择数据源分类!',
            },
          ]}>
          <Select
            allowClear
            options={
              sessionStorage.getItem('project_type') === '0'
                ? [
                    { label: '自添加数据源', value: 'datasource' },
                    { label: 'AIlab', value: 'ailab' },
                  ]
                : [{ label: 'AIlab', value: 'ailab' }]
            }
            onSelect={selectDsType}
            placeholder="请选择数据源分类"
          />
        </Form.Item>
        {dsType === 'datasource' && (
          <Form.Item
            label="选择数据源"
            name="datasource_name"
            rules={[
              {
                required: true,
                message: '请选择数据源!',
              },
            ]}>
            <Select
              showSearch
              optionFilterProp="label"
              options={datasources}
              onSelect={selectDatasource}
              placeholder="请选择数据源"
              allowClear
            />
          </Form.Item>
        )}
        <Form.Item
          label="选择表"
          name="datasource_table"
          rules={[
            {
              required: true,
              message: '请选择表',
            },
          ]}>
          <Select
            showSearch
            optionFilterProp="label"
            options={tableList}
            allowClear
            onSelect={selectTable}
            placeholder="请选择数据表"
          />
        </Form.Item>
        {datasourceType === 'mysql' ? (
          <>
            <Form.Item label="pre_sql" name="pre_sql" wrapperCol={{ span: 10 }}>
              <Input.TextArea rows={5} placeholder="请输入pre_sql语句" />
            </Form.Item>
            <Form.Item
              label="post_sql"
              name="post_sql"
              wrapperCol={{ span: 10 }}>
              <Input.TextArea rows={5} placeholder="请输入post_sql语句" />
            </Form.Item>
          </>
        ) : (
          <>
            <Form.Item
              label="选择路径"
              name="writer_path"
              wrapperCol={{ span: 10 }}
              rules={[
                {
                  required: true,
                  message: '请选择路径!',
                },
              ]}>
              <Input.TextArea rows={5} placeholder="要读取的文件路径" />
            </Form.Item>
            {dsType === 'datasource' && (
              <Form.Item
                label="HDFS"
                name="writer_default_fs"
                rules={[
                  {
                    required: true,
                    message: '请输入HDFS',
                  },
                ]}>
                <Input placeholder="Hadoop hdfs文件系统namenode节点地址" />
              </Form.Item>
            )}
            <Form.Item
              label="写入文件名"
              name="writer_filename"
              rules={[
                {
                  required: true,
                  message: '请输入文件名称',
                },
              ]}>
              <Input placeholder="请输入文件名称" />
            </Form.Item>
            <Form.Item
              label="文件类型"
              name="writer_file_type"
              rules={[
                {
                  required: true,
                  message: '请选择文件类型',
                },
              ]}>
              <Select
                options={[
                  { label: 'text', value: 'text' },
                  { label: 'csv', value: 'csv' },
                  { label: 'orc', value: 'orc' },
                ]}
                allowClear
                placeholder="文件的类型"
              />
            </Form.Item>
            <Form.Item
              label="写入模式"
              name="writer_mode"
              rules={[
                {
                  required: true,
                  message: '请选择写入模式',
                },
              ]}>
              <Select allowClear placeholder="写入模式" optionLabelProp="label">
                <Option label="append" value="append">
                  <Tooltip title="append：写入前不做任何处理，数据集成HDFS Writer直接使用filename写入，并保证文件名不冲突。">
                    <div>append</div>
                  </Tooltip>
                </Option>
                <Option label="nonConflict" value="nonConflict">
                  <Tooltip title="nonConflict：如果目录下有fileName前缀的文件，直接报错。">
                    <div>nonConflict</div>
                  </Tooltip>
                </Option>
                <Option label="truncate" value="truncate">
                  <Tooltip title="truncate：写入前清理fileName名称前缀匹配的所有文件。例如，'fileName': 'abc'，将清理对应目录所有abc开头的文件。">
                    <div>truncate</div>
                  </Tooltip>
                </Option>
              </Select>
            </Form.Item>
            <Form.Item
              label="分隔符"
              name="writer_field_delimiter"
              rules={[
                {
                  required: true,
                  message: '请输入分隔符!',
                },
              ]}>
              <Input placeholder="读取字段的分隔符" />
            </Form.Item>
          </>
        )}
      </Form>
    </>
  )
}
