import { Tabs, Button, Space, Select, Modal, Input, Form, message } from 'antd'
import DataSourceManage from '../component/DatasourceManage.jsx'
import DatasourceAdd from '../component/DatasourceAdd'
import DatasourceSyncView from '../component/DatasourceSyncView'
import DatasourceLog from '../component/DatasourceLog'
import React, { useState, useEffect } from 'react'
import { Link, useNavigate, useLocation } from 'react-router-dom'
import styled from 'styled-components'
import { importDatalake, craeteAilab } from '../services'
import { MinusCircleOutlined, PlusOutlined } from '@ant-design/icons'

const { TabPane } = Tabs

const Datasource = styled.div`
  padding: 20px;
`
const FormItem = Form.Item
const FormList = Form.List

export default function DatasourceView() {
  const navigate = useNavigate()
  const { state } = useLocation()
  // 数据源管理ref
  const sourceRef = React.useRef()
  const logRef = React.useRef()
  // 当前所处页面
  const [currentKey, setCurrentKey] = useState('1')
  const [currentLog, setCurrentLog] = useState(null)
  const [currentSyncPage, setCurrentSyncPage] = useState(1)
  const [currentSyncPageSize, setCurrentSyncPageSize] = useState(10)
  const [currentLogPage, setCurrentLogPage] = useState(1)
  const [currentLogPageSize, setCurrentLogPageSize] = useState(10)
  // 导入数据表
  const [importData, setImportData] = useState(false)
  const [isCreateTable, setIsCreateTable] = useState(false)
  const [datatableForm] = Form.useForm()
  const [ailabtableForm] = Form.useForm()
  // 数据隔离
  const [currentDataType, setCurrentDataType] = useState('ailab')

  useEffect(() => {
    if (state) {
      setCurrentKey(state.key)
      if (state.key === '2') {
        setCurrentSyncPage(state.currentPage)
        setCurrentSyncPageSize(state.currentPageSize)
      } else if (state.key === '3') {
        setCurrentLogPage(state.currentPage)
        setCurrentLogPageSize(state.currentPageSize)
        setCurrentLog(state.logId)
      }
    }
  }, [state])
  // 更新数据源列表
  const updateDataSource = () => {
    sourceRef.current?.updateSourceList()
  }

  const updateAilab = () => {
    sourceRef.current?.updateAilabList()
  }

  const updateLake = () => {
    sourceRef.current?.updateLakeList()
  }
  const updateLogs = () => {
    logRef.current?.updateLogList()
  }

  const selectLog = id => {
    setCurrentLog(id)
    setCurrentKey('3')
  }

  const changeDataType = val => {
    setCurrentDataType(val)
  }

  const importTable = () => {
    datatableForm
      .validateFields()
      .then(async () => {
        const { database_name, datatable_name, datatable_path } =
          datatableForm.getFieldValue()
        const { data } = await importDatalake({
          database_name,
          table_name: datatable_name,
          table_path: datatable_path,
        })
        if (data.code === 200) {
          message.success('导入成功')
          datatableForm.resetFields()
          setImportData(false)
          updateLake()
        } else {
          message.error(data.msg)
        }
      })
      .catch(err => {
        message.error('请检查表单数据是否完整')
      })
  }
  const importModal = (
    <Modal
      title="导入数据表"
      visible={importData}
      onOk={importTable}
      okText="确认导入"
      onCancel={() => {
        datatableForm.resetFields()
        setImportData(false)
      }}>
      <Form form={datatableForm}>
        <FormItem
          name="database_name"
          label="数据库名称"
          rules={[
            {
              required: true,
              message: '请输入数据库名称!',
            },
          ]}>
          <Input placeholder="输入数据库名称" />
        </FormItem>
        <FormItem
          name="datatable_name"
          label="数据表名称"
          rules={[
            {
              required: true,
              message: '请输入数据表名称!',
            },
          ]}>
          <Input placeholder="输入数据表名称" />
        </FormItem>
        <FormItem
          name="datatable_path"
          label="数据表路径"
          rules={[
            {
              required: true,
              message: '请输入数据表路径!',
            },
          ]}>
          <Input placeholder="输入数据表路径" />
        </FormItem>
      </Form>
    </Modal>
  )

  const createAilabTable = () => {
    ailabtableForm
      .validateFields()
      .then(async () => {
        const { database_field, table_name, partition_column } =
          ailabtableForm.getFieldsValue()
        const params = {
          project_id: sessionStorage.getItem('project_id'),
          table_name,
          partition_column,
          columns: database_field,
        }
        const { data } = await craeteAilab(params)
        if (data.code === 200) {
          message.success('创建成功')
          ailabtableForm.resetFields()
          setIsCreateTable(false)
          updateAilab()
        } else {
          message.error(data.msg)
        }
      })
      .catch(err => {
        message.error('请检查表单数据是否完整')
      })
  }

  const formItemLayout = {
    labelCol: {
      xs: { span: 24 },
      sm: { span: 6 },
    },
    wrapperCol: {
      xs: { span: 24 },
      sm: { span: 18 },
    },
  }
  const formItemLayoutWithOutLabel = {
    wrapperCol: {
      xs: { span: 24, offset: 0 },
      sm: { span: 18, offset: 6 },
    },
  }

  const createTableModal = (
    <Modal
      title="创建数据表"
      visible={isCreateTable}
      onOk={createAilabTable}
      okText="确认创建"
      onCancel={() => {
        ailabtableForm.resetFields()
        setIsCreateTable(false)
      }}>
      <Form
        form={ailabtableForm}
        {...formItemLayout}
        initialValues={{
          database_field: [{ column_name: null, Column_type: null }],
        }}>
        <FormItem
          name="table_name"
          label="数据表名称"
          rules={[
            {
              required: true,
              message: '请输入数据表名称!',
            },
          ]}>
          <Input placeholder="输入数据表名称" />
        </FormItem>
        <FormList
          name="database_field"
          rules={[
            {
              validator: async (_, database_field) => {
                if (!database_field || database_field.length < 1) {
                  return Promise.reject(new Error('最少存在一个字段'))
                }
              },
            },
          ]}>
          {(fields, { add, remove }, { errors }) => (
            <div style={{ maxHeight: '400px', overflow: 'auto' }}>
              {fields.map((field, index) => (
                <Form.Item
                  {...(index === 0
                    ? formItemLayout
                    : formItemLayoutWithOutLabel)}
                  label={index === 0 ? '表字段设置' : ''}
                  required={true}
                  key={field.key}>
                  <Space>
                    <Form.Item
                      {...field}
                      key={`filed_${field.key}`}
                      name={[field.name, 'column_name']}
                      validateTrigger={['onChange', 'onBlur']}
                      rules={[
                        {
                          required: true,
                          whitespace: true,
                          message: '请输入表字段',
                        },
                      ]}
                      noStyle>
                      <Input placeholder="输入表字段" />
                    </Form.Item>
                    <Form.Item
                      {...field}
                      key={`type_${field.key}`}
                      name={[field.name, 'Column_type']}
                      validateTrigger={['onChange', 'onBlur']}
                      rules={[
                        {
                          required: true,
                          whitespace: true,
                          message: '请选择表类型',
                        },
                      ]}
                      noStyle>
                      <Select
                        style={{ minWidth: '100px' }}
                        placeholder="选择表类型"
                        options={[
                          { value: 'tinyint', label: 'tinyint' },
                          { value: 'smallint', label: 'smallint' },
                          { value: 'int', label: 'int' },
                          { value: 'bigint', label: 'bigint' },
                          { value: 'boolean', label: 'boolean' },
                          { value: 'float', label: 'float' },
                          { value: 'double', label: 'double' },
                          { value: 'string', label: 'string' },
                          { value: 'binary', label: 'binary' },
                          { value: 'timestamp', label: 'timestamp' },
                          { value: 'decimal', label: 'decimal' },
                          { value: 'char', label: 'char' },
                          { value: 'varchar', label: 'varchar' },
                          { value: 'date', label: 'date' },
                        ]}
                      />
                    </Form.Item>
                    {fields.length > 1 ? (
                      <MinusCircleOutlined
                        className="dynamic-delete-button"
                        onClick={() => remove(field.name)}
                      />
                    ) : null}
                  </Space>
                </Form.Item>
              ))}
              <Form.Item {...formItemLayoutWithOutLabel}>
                <Button
                  type="dashed"
                  onClick={() => add()}
                  block
                  icon={<PlusOutlined />}>
                  继续添加
                </Button>
              </Form.Item>
            </div>
          )}
        </FormList>
        <FormItem name="partition_column" label="分区字段">
          <Input placeholder="输入分区字段" />
        </FormItem>
      </Form>
    </Modal>
  )

  // Tab标签右边按钮
  const tabExtraBtn = () => {
    switch (currentKey) {
      case '1':
        return (
          <Space>
            <Select
              value={currentDataType}
              onChange={changeDataType}
              options={
                ['1', '2', '3'].includes(sessionStorage.getItem('role'))
                  ? [
                      { value: 'datasource', label: '自添加数据源' },
                      { value: 'ailab', label: 'AIlab' },
                      { value: 'datalake', label: '数据湖' },
                    ]
                  : [{ value: 'ailab', label: 'AIlab' }]
              }
            />
            {currentDataType === 'datasource' &&
              sessionStorage.getItem('role') === '1' && (
                <DatasourceAdd updateDataSource={updateDataSource} />
              )}
            {currentDataType === 'ailab' && (
              <Button
                type="primary"
                onClick={() => {
                  setIsCreateTable(true)
                }}>
                创建数据表
              </Button>
            )}
            {currentDataType === 'datalake' &&
              sessionStorage.getItem('role') === '1' && (
                <Button
                  type="primary"
                  onClick={() => {
                    setImportData(true)
                  }}>
                  导入数据表
                </Button>
              )}
          </Space>
        )
      case '2':
        return (
          <Link to="/datasource/sync-task">
            <Button type="primary">添加同步任务</Button>
          </Link>
        )
      default:
        return
    }
  }
  return (
    <Datasource>
      {/* Tab栏及内容 */}
      <Tabs
        type="card"
        tabBarExtraContent={tabExtraBtn()}
        activeKey={currentKey}
        onTabClick={key => {
          if (key === '3') updateLogs()
          navigate('/datasource', { state: { key } })
        }}
        onChange={key => {
          setCurrentKey(key)
          setCurrentLog(null)
        }}>
        <TabPane tab="数据源管理" key="1">
          <DataSourceManage ref={sourceRef} dataType={currentDataType} />
        </TabPane>
        <TabPane tab="同步配置" key="2">
          <DatasourceSyncView
            selectLog={id => selectLog(id)}
            page={currentSyncPage}
            page_size={currentSyncPageSize}
          />
        </TabPane>
        <TabPane tab="日志管理" key="3">
          <DatasourceLog
            onRef={logRef}
            logId={currentLog}
            page={currentLogPage}
            page_size={currentLogPageSize}
          />
        </TabPane>
      </Tabs>
      {importModal}
      {createTableModal}
    </Datasource>
  )
}
