import React, { useState, useEffect } from 'react'
import { Table, Space, message } from 'antd'
import moment from 'moment'
import { useNavigate, useLocation } from 'react-router-dom'
import { getLogList, refreshLogsStatus } from '../services/index'
import styled from 'styled-components'

const LogList = styled.div``

let timeout = null

const LogTable = () => {
  // 初始化日志列表
  const [logList, setLogList] = useState([])
  // 表格Loading状态
  const [dataLoading, setDataLoading] = useState(false)

  const [currentPage, setCurrentPage] = useState(1)
  const [currentPageSize, setCurrentPageSize] = useState(10)
  const [dataTotal, setDataTotal] = useState(0)

  const navigate = useNavigate()

  const { state } = useLocation()

  const checkLog = id => {
    navigate('/task-log/log-watcher', {
      state: { id, logId: state?.id, currentPage, currentPageSize },
    })
  }
  const columns = [
    {
      title: '任务名称',
      dataIndex: 'jobName',
      key: 'jobName',
    },
    {
      title: '任务类型',
      dataIndex: 'taskType',
      key: 'taskType',
    },
    {
      title: '任务标签',
      dataIndex: 'taskTag',
      key: 'taskTag',
    },
    {
      title: '开始时间',
      dataIndex: 'startTime',
      key: 'startTime',
    },
    {
      title: '任务状态',
      dataIndex: 'handleResult',
      key: 'handleResult',
      render: code => (
        <span
          style={{
            color: code === 3 ? '#FF4D4F' : code === 2 ? '#52C41A' : '#4A4A4A',
          }}>
          {code === 3
            ? '失败'
            : code === 2
            ? '成功'
            : code === 1
            ? '运行中'
            : code === 0
            ? '队列中'
            : '暂无'}
        </span>
      ),
    },
    {
      title: '操作',
      key: 'operation',
      render: (_, record) => (
        <Space size="middle">
          <span
            onClick={() => {
              checkLog(record.key)
            }}
            style={{ color: '#1890ff', cursor: 'pointer' }}>
            查看
          </span>
          {/* <span
            onClick={() => {
              changeTask(record.key)
            }}
            style={{ color: '#1881DA', cursor: 'pointer' }}>
            终止任务
          </span> */}
        </Space>
      ),
    },
  ]

  const fetchJoblog = async (page = 1, pageSize = 10) => {
    setDataLoading(true)
    const { data } = await getLogList({
      page: page,
      size: pageSize,
      id: state?.id,
    })
    if (data.code === 200) {
      setDataTotal(data.data.extra_data.total)
      setCurrentPage(data.data.extra_data.page)
      setCurrentPageSize(data.data.extra_data.size)
      const refreshLogs = []
      const list = data.data.item.map(item => {
        if ([0, 1].includes(item.execute_result)) {
          refreshLogs.push(item.id)
        }
        return {
          key: item.id,
          jobName: item.job_name,
          startTime: moment(Math.round(item.start_time) * 1000).format(
            'YYYY.MM.DD HH:mm:ss'
          ),
          handleResult: item.result,
          taskType: item.job_type,
          taskTag: item.job_tag,
          afJobId: item.af_job_id,
          runId: item.run_id,
        }
      })
      setLogList(list)
      refreshStatus(refreshLogs)
    }
    setDataLoading(false)
  }

  const refreshStatus = async refreshLogs => {
    if (refreshLogs.length !== 0) {
      const { data } = await refreshLogsStatus(refreshLogs.toString())
      if (data.code === 200) {
        const list = []
        let logs = []
        const keys = Object.keys(data.data)
        keys.forEach(item => {
          if ([2, 3].includes(data.data[item])) {
            logs = logList.map(logData => {
              if (logData.id === item) {
                logData.execute_result = data.data[item]
              }
              return logData
            })
          } else {
            list.push(item)
          }
        })
        if (logs.length !== 0) {
          setLogList(logs)
        }
        timeout = setTimeout(() => {
          refreshStatus(list)
        }, 10000)
      } else {
        message.error(data.msg)
      }
    }
  }

  const pageChange = (page, pageSize) => {
    fetchJoblog(page, pageSize)
  }

  useEffect(() => {
    fetchJoblog()
  }, [])

  useEffect(() => {
    fetchJoblog()
  }, [sessionStorage.getItem('project_id')])

  useEffect(() => {
    if (state) {
      fetchJoblog(state.currentPage || 1, state.currentPageSize || 10)
    }
  }, [state])

  useEffect(() => {
    return () => {
      clearTimeout(timeout)
    }
  }, [])

  return (
    <LogList>
      <Table
        columns={columns}
        dataSource={logList}
        bordered
        loading={dataLoading}
        pagination={{
          current: currentPage,
          total: dataTotal,
          pageSize: currentPageSize,
          onChange: pageChange,
        }}
      />
    </LogList>
  )
}

export default LogTable
