import { Button, Card, Checkbox, Input, message, Space, Table } from 'antd'
import React, { useEffect, useState } from 'react'
import { useLocation, useNavigate } from 'react-router-dom'
import styled from 'styled-components'
import { getAllTask, getTaskLog } from '../services/index'
import moment from 'moment'
import { UpOutlined, DownOutlined } from '@ant-design/icons'
import Mark from 'mark.js'

const LogWrapper = styled.div`
  padding: 20px;
  display: flex;
  height: 100vh;
  column-gap: 20px;
  .job-list {
    flex: 1;
  }
  .log-info {
    width: 70%;
  }
  .refresh {
    height: 50px;
    display: flex;
    justify-content: space-between;
  }
  .log {
    background-color: #012b36;
    height: calc(100% - 50px);
    width: 100%;
    overflow-y: scroll;
    color: #638691;
    padding: 8px 10px;
    ::-webkit-scrollbar {
      display: none;
    }
  }

  .node_list {
    width: 100%;
    height: 100%;
  }
  .node_table {
    overflow: auto;
    height: 650px
    ::-webkit-scrollbar {
      display: none;
    }
  }
  .ant-table-selection-column {
    display: none !important;
  }
`

let timeout = null

const TaskLogWatcher = () => {
  const { state } = useLocation()
  const navigate = useNavigate()
  const [logData, setLogData] = useState('暂无日志')
  // 初始化节点列表
  const [jobList, setJobList] = useState([])
  // 选中的作业
  const [selectJob, setSelectJob] = useState({})
  // 表格Loading状态
  const [dataLoading, setDataLoading] = useState(false)

  const [autoRefresh, setAutoRefresh] = useState(false)

  const [keyWord, setKeyWord] = useState(null)

  const [keyWordList, setKeyWordList] = useState([])

  const [keyWordIndex, setKeyWordIndex] = useState(0)

  const columns = [
    {
      title: '序号',
      dataIndex: 'index',
      key: 'index',
    },
    {
      title: '作业名称',
      dataIndex: 'name',
      key: 'name',
    },
    {
      title: '执行时间',
      dataIndex: 'executeTime',
      key: 'executeTime',
    },
    {
      title: '执行结果',
      dataIndex: 'handleResult',
      key: 'handleResult',
      render: code => (
        <span
          style={{
            color: code === 3 ? '#FF4D4F' : code === 2 ? '#52C41A' : '#4A4A4A',
          }}>
          {code === 3
            ? '失败'
            : code === 2
            ? '成功'
            : code === 1
            ? '运行中'
            : '队列中'}
        </span>
      ),
    },
  ]

  const fetchAllTask = async () => {
    setDataLoading(true)
    const { data } = await getAllTask(state.id)
    if (data.code === 200) {
      formatSingleTask(data.data)
    } else {
      message.error(data.msg)
    }
    setDataLoading(false)
  }

  const formatSingleTask = data => {
    const list = data.map((item, index) => ({
      key: index,
      id: item.id,
      index: index + 1,
      executeTime: moment(Math.round(item.execute_time) * 1000).format(
        'YYYY.MM.DD HH:mm:ss'
      ),
      name: item.name,
      log: item.log,
      handleResult: item.execute_result,
      task_id: item.task_id,
      job_id: item.job_id,
      af_run_id: item.af_run_id,
    }))
    setJobList(list)
    onClickTableNode(list[0])
  }

  const onClickTableNode = async record => {
    setSelectJob(record)
    fetchTaskLog(record)
  }

  const fetchTaskLog = async record => {
    const params = {
      job_id: record.job_id,
      task_id: record.task_id,
      af_run_id: record.af_run_id,
    }
    const { data } = await getTaskLog(params)
    if (data.code === 200) {
      setLogData(data.data.log)
      if (![2, 3].includes(data.data.status) && autoRefresh) {
        timeout = setTimeout(() => {
          fetchTaskLog(params)
        }, 5000)
      } else {
        const index = jobList.findIndex(item => item.key === record.key)
        const list = [...jobList]
        if (list[index]) {
          list[index].handleResult = data.data.status
          setJobList(list)
        }
      }
    } else {
      message.error(data.msg)
    }
  }

  const changeRefresh = e => {
    setAutoRefresh(e.target.checked)
  }

  useEffect(() => {
    if (autoRefresh) {
      fetchTaskLog(selectJob)
    } else {
      clearTimeout(timeout)
    }
  }, [autoRefresh])

  const onClickRefresh = () => {
    fetchTaskLog(selectJob)
  }

  const returnLogTable = () => {
    navigate('/task-log', {
      state: {
        currentPage: state.currentPage,
        currentPageSize: state.currentPageSize,
        id: state.logId,
      },
    })
  }

  useEffect(() => {
    fetchAllTask()
  }, [state])

  useEffect(() => {
    return () => {
      clearTimeout(timeout)
    }
  }, [])

  const singleTable = (
    <Table
      columns={columns}
      dataSource={jobList}
      showHeader={false}
      className="node_table"
      loading={dataLoading}
      pagination={{ position: ['none', 'none'] }}
      onRow={record => {
        return {
          onClick: () => onClickTableNode(record),
        }
      }}
      rowSelection={{
        type: 'radio',
        columnWidth: '0',
        selectedRowKeys: [selectJob.key],
      }}
    />
  )
  const findLast = () => {
    if (keyWordList.length) {
      const logWindow = document.getElementById('log')
      keyWordList[keyWordIndex - 1].style.backgroundColor = '#fffd38'
      if (keyWordIndex - 1 > 0) {
        setKeyWordIndex(keyWordIndex - 1)
        keyWordList[keyWordIndex - 2].style.backgroundColor = '#fd9640'
        logWindow.scrollTop = keyWordList[keyWordIndex - 2].offsetTop - 250
        logWindow.scrollLeft =
          keyWordList[keyWordIndex - 2].offsetLeft - logWindow.clientWidth + 150
      } else {
        setKeyWordIndex(keyWordList.length)
        keyWordList[keyWordList.length - 1].style.backgroundColor = '#fd9640'
        logWindow.scrollTop =
          keyWordList[keyWordList.length - 1].offsetTop - 250
        logWindow.scrollLeft =
          keyWordList[keyWordList.length - 1].offsetLeft -
          logWindow.clientWidth +
          150
      }
    }
  }
  const findNext = () => {
    if (keyWordList.length) {
      const logWindow = document.getElementById('log')
      keyWordList[keyWordIndex - 1].style.backgroundColor = '#fffd38'
      if (keyWordIndex + 1 <= keyWordList.length) {
        setKeyWordIndex(keyWordIndex + 1)
        keyWordList[keyWordIndex].style.backgroundColor = '#fd9640'
        logWindow.scrollTop = keyWordList[keyWordIndex].offsetTop - 250
        logWindow.scrollLeft =
          keyWordList[keyWordIndex].offsetLeft - logWindow.clientWidth + 150
      } else {
        setKeyWordIndex(1)
        keyWordList[0].style.backgroundColor = '#fd9640'
        logWindow.scrollTop = keyWordList[0].offsetTop - 250
        logWindow.scrollLeft =
          keyWordList[0].offsetLeft - logWindow.clientWidth + 150
      }
    }
  }
  const changeKeyWord = e => {
    setKeyWord(e.target.value)
    setKeyWordIndex(0)
    const markInstance = new Mark(document.querySelector('#log'))
    markInstance.unmark({
      done: () => {
        const list = []
        markInstance.mark(e.target.value, {
          each: val => {
            val.style.backgroundColor = '#fffd38'
            val.style.padding = 0
            list.push(val)
          },
        })
        if (list.length > 0) {
          setKeyWordIndex(1)
          const logWindow = document.getElementById('log')
          list[0].style.backgroundColor = '#fd9640'
          logWindow.scrollTop = list[0].offsetTop - 250
          logWindow.scrollLeft =
            list[0].offsetLeft - logWindow.clientWidth + 150
        }
        setKeyWordList(list)
      },
    })
  }
  const searchTools = (
    <Space>
      <span>
        {keyWordIndex}/{keyWordList.length}
      </span>
      <UpOutlined onClick={findLast} />
      <DownOutlined onClick={findNext} />
    </Space>
  )
  return (
    <LogWrapper>
      <div className="job-list">
        <Card
          size="small"
          title="节点列表"
          bordered={false}
          className="node_list"
          headStyle={{
            padding: '10px 15px',
            fontSize: '14px',
            fontWeight: '700',
          }}>
          {singleTable}
        </Card>
      </div>
      <div className="log-info">
        <Card
          bordered={false}
          style={{ height: '100%', width: '100%' }}
          bodyStyle={{ height: '100%', width: '100%' }}>
          <div className="refresh">
            <Space>
              <Input
                value={keyWord}
                onChange={changeKeyWord}
                addonAfter={searchTools}
                placeholder="输入关键词"
              />
            </Space>
            <Space>
              <Checkbox onChange={changeRefresh} checked={autoRefresh}>
                自动刷新
              </Checkbox>
              <Button onClick={onClickRefresh}>刷新</Button>
              <Button
                type="primary"
                onClick={() => {
                  returnLogTable()
                }}>
                返回上一级
              </Button>
            </Space>
          </div>
          <pre className="log" id="log">
            {logData}
          </pre>
        </Card>
      </div>
    </LogWrapper>
  )
}

export default TaskLogWatcher
