import { message } from "antd";
import axios from "axios";

const axiosInstance = axios.create({
  baseURL: process.env.REACT_APP_BASE_URL,
  timeout: 1200000,
  headers: {
    'Content-Type': 'application/json;charset=UTF-8',
    Accept: 'application/json',
  }
})

// 请求拦截器
axiosInstance.interceptors.request.use((config) => {
  config.headers['user-token'] = sessionStorage.getItem('user_token')
  config.headers['item-token'] = sessionStorage.getItem('project_token')
  return config
})

//响应拦截器
axiosInstance.interceptors.response.use(
  response => {
    if (response.status === 200) {
      return Promise.resolve(response)
    } else {
      return Promise.reject(response);
    }
  },
  error => {
    if (error.response.status) {
      switch (error.response.status) {
        case 401:
          sessionStorage.clear()
          window.location.pathname = "/login"
          break
        case 403:
          error.response.data = {
            code: 404,
            msg: '请选择项目'
          }
          break
        default:
          error.response.data = {
            code: 404,
            msg: '接口错误'
          }
          break
      }
      return Promise.resolve(error.response);
    }
  }
)

export default axiosInstance